# Padrões Desenvolvimento - HCM Services

## *Back-end*

### Limite de Conceitos

Uma classe de determinado assunto (*controller*, *service*, *repository* ou *logic*) deverá apenas manipular informações do seu conceito. A manipulação de informações que fogem desse limite deverá ser de responsabilidade de outra classe.

### Exceções

As rotinas implementadas devem ser orientadas à exceções. Quaisquer situações que fujam do controle principal da execução devem ser tratadas com a criação de exeções.

```php
if (empty($data)) {
    throw new \Exception(Messages::MSG_ALERT);
}
```

### Tratativa de Erros nas Requisições

Nos *controllers*, acrescentar *catches* para capturar exceções previstas e exeções gerais.

```php
public function setData(
        Request\Row $request,
        Response    $response
) {
    try {
        $row = (array) $request->getRow()->getArrayCopy();
        
        [...]

        $dataset = [
            'success' => $success,
        ];

        $response->addDataSet(new DataSet('dataSet', $dataset));
    } catch (ExpectedException $e) {
        $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
    } catch (\Exception $e) {
        $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
    }
}
```

### Validações

Quaisquer validações realizadas em rotinas deverão ser executadas no início da mesma e, caso falhem, o fluxo deverá ser desviado por meio de um `return` levantamento de exceção.

```php
public function efetivar() {
    $itens = $this->getItensAssociativeArray();
    $data = $itens['DATA'] ?? null;

    if (empty($data)) {
        throw new \Exeption();
    }

    ...
}
```

### Geração de PKs

Deverá ser utilizado sempre o `Util\NovoCodigo`, por meio de instruções como o exemplo abaixo:

```php
NovoCodigo::geraCodigo($nrOrg, 'GPE_VINCULOM', 1, 12, true, false)
```

### Comandos SQL

A utilização de comandos ou instruções SQL deverá ficar restrita aos *repositories* ou aos repositórios das entidades.

### Strings de Mensagens

Mensagens de alerta ou avisos que são retornadas nas requisições deverão ser armazenadas em `Messages`.

### Tipagem

Todas as novas funções criadas ou alteradas deverão ter tipos nos parâmetros e no retorno, seguindo o padrão do [PHP](https://www.php.net/manual/en/language.types.declarations.php). Parâmetros que podem ser nulos deverão ser sinalizados com o `?`.

```php
public function incluiValidacaoItens(array $dadosRegistro): array;
```

### Documentação de Funções

Todas as novas funções criadas ou alteradas deverão ser documentadas seguindo o padrão [javadoc](https://pt.wikipedia.org/wiki/Javadoc).

```php
/**
 * A partir de uma lista de itens adiciona o valor de validacao ao final de cada linha
 * @param array $dadosRegistro
 * @return array
 */
public function incluiValidacaoItens(array $dadosRegistro): array;
```
### Constantes

Sempre que possível, definir valores fixos utilizados em mais de um ponto em constantes.

```php
const SEGUNDA = 1;

if ($this->diaSemanaData == self::SEGUNDA) {
    $this->refAuxSaldos['DSR_DESCONTADO'] = [];
}
```

### Manipulação de Dados

Para realizar operações de manipulação de dados, sugere-se que o *service* extenda informações da classe `Ems\Util\Service`, e utilize as funções `set`, `update` e `remove`.

```php
class SolicitacaoService extends Service {
    public function getEntityRepositoryName(): string {
        return Repositories::GPE_SOLICITACAO;
    }

    public function getEntityKeysName(): array {
        return [
            'NRORG',
            'NRSOLICITACAO'
        ];
    }

    public function atualizaSolicitacao() {
        [...]

        $this->update(
            [
                'NRORG'             => $this->environment->getNrorg(),
                'NRSOLICITACAO'     => $nrSolicitacao,
                'IDAPROVACAOSEC'    => $idaprovacao,
                'CDOPERULTATU'      => $cdOperadorAprovante,
            ]
        );
        [...]
    }
}
```

É possível utilizá-la fora de um *service*, quando necessário.

```php
$this->getEntityPersistInstance()->setNewEntity(
    'REPOSITORY_NAME',
    $row,
    $config
);
```

### Boas Práticas
- Utilizar `empty` pra verificação de existência de valor em variável